<?php

namespace App\Http\Controllers\Campaign;

use App\Campaign;
use App\CampaignWitness;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class CampaignPhotographicWitnessesController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:witnesses.index')->only('index');
        $this->middleware('can:witnesses.indexWitnesses')->only('indexWitnesses');
        $this->middleware('can:witnesses.searchWitnesses')->only('searchWitnesses');
        $this->middleware('can:witnesses.store')->only('store');
        $this->middleware('can:witnesses.update')->only('update');
        $this->middleware('can:witnesses.destroy')->only('destroy');
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Campaign $campaign)
    {
        $witnesses = $campaign->witnesses;
        return $this->showAll( Collection::make($witnesses) );
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function indexWitnesses()
    {
        $witnesses = CampaignWitness::orderBy( 'id', 'desc' )
            ->with('campaign:id,name')
            ->paginate(8);

        return $this->showAll(Collection::make($witnesses));
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function searchWitnesses()
    {
        $rules = [
            'word'    =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $witnesses = CampaignWitness::select( 'campaign_witnesses.*', 'c.id as campaignId', 'c.name' )
            ->orderBy( 'c.id', 'desc' )
            ->join('campaigns as c','c.id','=','campaign_witnesses.campaign_id')
            ->orWhere( 'campaign_witnesses.provider_wall_id', request('word') )
            ->orWhere('campaign_witnesses.provider_wall_id', 'LIKE', '%'.request('word').'%')
            ->orWhere( 'c.name', request('word') )
            ->orWhere('c.name', 'LIKE', '%'.request('word').'%')
            ->paginate(8);

        return $this->showAll(Collection::make($witnesses));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Campaign $campaign)
    {
        $img = request()->file( 'file' );

        $input = [
            'image' => $img
        ];

        $rules = [
            'image' => ['required', 'image']
        ];

        $validate = Validator::make($input, $rules);

        if ($validate->fails())
        {
            return $validate->errors();
        }

        $data['image'] = request('file')->store('/'.$campaign->id, 'campaign_witnesses');

        $witness = CampaignWitness::create([
            'campaign_id' => $campaign->id,
            'img' => $data['image']
        ]);

        return $this->responseUploadFile( $witness, 201 );
    }

    /**
     * Update resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function update(CampaignWitness $witness)
    {
        $witness->fill(request()->only(
            'provider_wall_id', 'description'
        ));

        $witness->save();

        return $this->showOne($witness);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Campaign  $campaign
     * @return \Illuminate\Http\Response
     */
    public function destroy(CampaignWitness $image)
    {
        Storage::disk('campaign_witnesses')->delete('/' . $image->img );
        $image->delete();
        return $this->showOne( $image );
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function downloadZip(Campaign $campaign)
    {
        $files = glob('img/witnesses/' . $campaign->id );
        $file = \Madnest\Madzipper\Facades\Madzipper::make('files/temporary/zips/'. $campaign->id .'.zip')->add($files)->close();
        $data['zip'] = $campaign->id . '.zip';
        return $this->responseUploadFile( $data, 201 );
    }
}
