<?php

namespace App\Http\Controllers\Company;
use App\Campaign;
use App\Company;
use App\Country;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\Provider;
use App\Reseller;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class CompanyController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:companies.index')->only('index');
        $this->middleware('can:companies.administratorCompanies')->only('administratorCompanies');
        $this->middleware('can:companies.providerCompanies')->only('providerCompanies');
        $this->middleware('can:companies.resellerCompanies')->only('resellerCompanies');
        $this->middleware('can:companies.clientCompanies')->only('clientCompanies');
        $this->middleware('can:companies.show')->only('show');
        $this->middleware('can:companies.store')->only('store');
        $this->middleware('can:companies.update')->only('update');
        $this->middleware('can:companies.uploadImage')->only('uploadImage');
        $this->middleware('can:companies.deleteImage')->only('deleteImage');
        $this->middleware('can:companies.deleteCompanyImage')->only('deleteCompanyImage');
        $this->middleware('can:companies.setMultiplierProvider')->only('setMultiplierProvider');
        $this->middleware('can:companies.getMultiplierProvider')->only('getMultiplierProvider');
        $this->middleware('can:companies.getMultiplierProvidersCampaign')->only('getMultiplierProvidersCampaign');
        $this->middleware('can:companies.search')->only('search');
    }

    /**
     * Metodo para listar todos las empresas
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Resellers
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if (request()->get('per_page') === 'all') {
            $companies = Company::select('id', 'name_company')->orderBy( 'id', 'desc' )->get();
            return $this->showAll( Collection::make($companies) );
        }

        $companies = Company::orderBy( 'id', 'desc' )
                             ->with('country')
                             ->with('countries')
                             ->paginate(8);

        return $this->showAll(Collection::make($companies));
    }

    /**
     * Metodo para buscar un usuario
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Resellers
     *
     * @return \Illuminate\Http\Response
    */
    public function search()
    {
        $rules = [
            'word'    =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $companies = Company::orderBy('id', 'DESC')
                         ->with('country')
                         ->orWhere('id', 'LIKE', '%'.request('word').'%')
                         ->orWhere('name_company', 'LIKE', '%'.request('word').'%')
                         ->paginate( 8 );

        return $this->showAll(Collection::make($companies), 201);
    }

    /**
     * Listar todos las empresas de tipo administrador.
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function administratorCompanies() {
        $companies = Company::where( 'role', Company::ADMINISTRATOR )
                            ->paginate( 12 );
        return $this->showAll( Collection::make($companies) );
    }

    /**
     * Listar todos las empresas de tipo proveedor.
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function providerCompanies() {

        $companies;

        if (count(auth()->user()->company->suppliers) != 0) {
            if (request()->get('per_page') === 'all') {
                $companies = auth()->user()->company->suppliers;
            } else {
                $companies = auth()->user()->company->suppliers()->paginate( 12 );
            }
        } else {
            if (request()->get('per_page') === 'all') {
                $companies = Company::where('role', Company::PROVIDER)->get();
            } else {
                $companies = Company::where( 'role', Company::PROVIDER )->paginate( 12 );
            }
        }

        return $this->showAll( Collection::make($companies) );
    }

    /**
     * Listar todos las empresas de tipo reseller.
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function resellerCompanies() {
        $companies = Company::where( 'role', Company::RESELLER )
                            ->paginate( 12 );
        return $this->showAll( Collection::make($companies) );
    }

    /**
     * Listar todos las empresas de tipo cliente.
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Resellers
     *
     * @return \Illuminate\Http\Response
    */
    public function clientCompanies() {
        $companies = Company::where( 'role', Company::CLIENT )
                            ->paginate( 12 );
        return $this->showAll( Collection::make($companies) );
    }

    /**
     * Crear empresas.
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Resellers
     *
     * @return \Illuminate\Http\Response
    */
    public function store()
    {
        $rules = [
            'name_company'      => ['required', 'string', 'max:255'],
            'country_id'        => ['required', 'integer'],
            'countries'         => ['array'],
            'countries.*.country_id' => ['required', 'numeric'],
            'countries.*.discount'   => ['required', 'numeric'],
            'countries.*.increase'   => ['required', 'numeric'],
            'companies.*'  => ['required', 'numeric'],
            'cell_phone'        => ['string', 'min:5', 'max:20'],
            'phone_number'      => ['string', 'min:5', 'max:20'],
            'fiscal_key'        => ['string', 'max:255'],
            'fiscal_address'    => ['string', 'max:255'],
            'bussiness_name'    => ['string', 'max:255'],
            'billing_email'     => ['string', 'email', 'max:255'],
            'role'              => [
                                    'required',
                                    'string', 'max:100',
                                    'in:administrator,client,reseller,provider'
                                ],
            'url_img'           => ['string'],
            'contact_name'      => ['string', 'min:1'],
            'contact_phone'     => ['string', 'min:1'],
            'contact_email'     => ['string', 'email'],
            'contact_position'  => ['string', 'min:1']
        ];

        $this->validate(request(), $rules);

        if (
            auth()->user()->hasRole('salesmanager') || auth()->user()->hasRole('sellercosts') ||
            auth()->user()->hasRole('seller')
        ) {
            if (request('role') === Company::ADMINISTRATOR || request('role') === Company::PROVIDER ) {
                return $this->errorResponse('You dont have enough privileges', 403);
            }
        }

        $company = Company::create(request()->all());

        // Agregar discount default 0
        if (request( 'countries' )) {
            foreach (request( 'countries' ) as $key => $countryFE) {
                if (Country::find($countryFE['country_id'])) {
                    $company->countries()
                            ->attach(
                                $countryFE['country_id'], ['discount' => $countryFE['discount'], 'increase' => $countryFE['increase']]
                            );
                }
            }
        }

        if (request( 'companies' )) {
            foreach (request( 'companies' ) as $key => $company_id) {
                if (Company::find($company_id)) {
                    $company->suppliers()
                        ->syncWithoutDetaching($company_id);
                }
            }
        }

        return $this->showOne($company, 201);
    }

    /**
     * Asignar Porcentajes.
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function setMultiplierProvider(Company $company)
    {
        $rules = [
            'multiplier'   => ['required', 'numeric']
        ];

        if ($company->role !== Company::PROVIDER ) {
            return $this->errorResponse('Error, invalid resource', 403);
        }

        $this->validate(request(), $rules);

        Provider::where('company_id', $company->id)->delete();

        $provider = Provider::create([
            'company_id' => $company->id,
            'multiplier' => request( 'multiplier' )
        ]);
        return $this->showOne($provider, 201);
    }

    /**
     * Asignar Porcentajes.
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Serllercosts
        ** Customercosts
     *
     * @return \Illuminate\Http\Response
    */
    public function getMultiplierProvider(Company $company)
    {
        $multiplier = Provider::where('company_id', $company->id)->orderby('created_at','DESC')->take(1)->get();
        return $this->showOne( $multiplier );
    }

    /**
     * Asignar Porcentajes a empresa reseller.
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function setMultiplierReseller(Company $company)
    {
        $rules = [
            'multiplier'   => ['required', 'numeric']
        ];

        if ($company->role !== Company::RESELLER ) {
            return $this->errorResponse('Error, invalid resource', 403);
        }

        $this->validate(request(), $rules);

        Reseller::where('company_id', $company->id)->delete();

        $provider = Reseller::create([
            'company_id' => $company->id,
            'multiplier' => request( 'multiplier' )
        ]);
        return $this->showOne($provider, 201);
    }

    /**
     * Asignar Porcentajes.
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Serllercosts
        ** Customercosts
     *
     * @return \Illuminate\Http\Response
    */
    public function getMultiplierReseller(Company $company)
    {
        $multiplier = Reseller::where('company_id', $company->id)->orderby('created_at','DESC')->take(1)->get();
        return $this->showOne( $multiplier );
    }

    /*
        Obtener arreglo de empresas con su multiplicador de una campaña
            Roles
            ** Superadministrator
            ** Administrator
            ** Salesmanaget
            ** Sellercosts
            ** Customercosts
    */
    public function getMultiplierProvidersCampaign(Campaign $campaign) {
        $screens_campaign = $campaign->screens->load('country')->load('circuit');

        $companiesIds = [];
        foreach ($screens_campaign as $key => $screen) {
            if (!in_array($screen->company_id, $companiesIds)) {
                array_push($companiesIds, $screen->company_id);
            }
        }

        $companiesMultiplier = Provider::whereIn( 'company_id', $companiesIds )->get();

        return $this->showOne($companiesMultiplier);
    }

    /**
     * Mostrar una empresas.
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Resellers
        ** Customers
     *
     * @return \Illuminate\Http\Response
    */
    public function show(Company $company)
    {
        return $this->showOne($company);
    }

    /**
     * Mostrar los paises de una empresa con su descuento.
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Resellers
        ** Customers
     *
     * @return \Illuminate\Http\Response
    */
    public function showCompanyPercentage(Company $company)
    {
        $countries_percentage = ['countries' => []];
        foreach ($company->countries as $key => $country) {
            array_push($countries_percentage['countries'],
                [
                    'id' => $country->id,
                    'country_name' => $country->country_name,
                    'discount'     => $country->pivot->discount,
                    'increase'     => $country->pivot->increase
                ]
            );
        }
        return $this->showAll(Collection::make($countries_percentage));
    }

    /**
     * Mostrar los paises de una empresa con su descuento.
     ** Superadministrators
     ** Administrators
     ** Salesmanager
     ** Resellers
     ** Customers
     *
     * @return \Illuminate\Http\Response
     */
    public function showCompanySuppliers(Company $company)
    {
        return $this->showAll(Collection::make($company->suppliers()->pluck('companies.id')));
    }

    /**
     * Actualizar una empresa.
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Resellers
     *
     * @return \Illuminate\Http\Response
    */

    public function update(Company $company)
    {
        $rules = [
            'name_company'   => ['string', 'max:255'],
            'country_id'     => ['integer'],
            'countries'      => ['array'],
            'countries.*.country_id' => ['required', 'numeric'],
            'countries.*.discount'   => ['required', 'numeric'],
            'countries.*.increase'   => ['required', 'numeric'],
            'companies.*'    => ['required', 'numeric'],
            'cell_phone'     => ['string', 'min:5', 'max:20'],
            'phone_number'   => ['string', 'min:5', 'max:20'],
            'fiscal_key'     => ['string', 'max:255'],
            'fiscal_address' => ['string', 'max:255'],
            'bussiness_name' => ['string', 'max:255'],
            'billing_email'  => ['string', 'email', 'max:255'],
            'role'           => [
                                    'string', 'max:100',
                                    'in:administrator,client,reseller,provider'
                                ],
            'url_img'    => ['string'],
            'contact_name'      => ['string', 'min:1'],
            'contact_phone'     => ['string', 'min:1'],
            'contact_email'     => ['string', 'email'],
            'contact_position'  => ['string', 'min:1']
        ];

        $this->validate(request(), $rules);

        $company->fill(request()->only([
            'name_company',
            'country_id',
            'cell_phone',
            'phone_number',
            'fiscal_key',
            'fiscal_address',
            'bussiness_name',
            'billing_email',
            'role',
            'url_img',
            'contact_name',
            'contact_phone',
            'contact_email',
            'contact_position',
        ]));

        if (request('countries')) {
            $company->countries()->detach();
            foreach (request( 'countries' ) as $key => $countryFE) {
                if (Country::find($countryFE['country_id'])) {
                    $company->countries()
                            ->attach(
                                $countryFE['country_id'], ['discount' => $countryFE['discount'], 'increase' => $countryFE['increase']]
                            );
                }
            }
        } else {
            $company->countries()->detach();
        }

        if (request( 'companies' )) {
            $company->suppliers()->detach();
            foreach (request( 'companies' ) as $key => $company_id) {
                if (Company::find($company_id)) {
                    $company->suppliers()
                        ->syncWithoutDetaching($company_id);
                }
            }
        } else {
            $company->suppliers()->detach();
        }


        $company->save();

        return $this->showOne($company);
    }

    /**
     * Subir imagen de la empresa
        -- Sube la imagen, pero se guardara en el momento que se cree o actualize un usuario
        ROLE
        ** Superadministradores
        ** Administradores
        ** Salesmanager
        ** Vendedores
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function uploadImage() {

        $img = request()->file( 'file' );

        $input = array( 'image' => $img );
        $rules = [ 'image' => ['required', 'image'] ];

        $validate = Validator::make($input, $rules);

        if ($validate->fails())
        {
            return $this->errorResponse('Only Images', 422);
        }

        $data['image'] = request('file')->store('', 'img_companies');

        return $this->responseUploadFile( $data, 201 );
    }

    /**
     * Solo deberia funcionar para eliminar en cambio de componente sin guardar
     * Eliminar imagen
        ROLE
        ** Superadministradores
        ** Administradores
        ** Salesmanager
        ** Vendedores
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteImage() {

        $rules = [
            'img'        =>    ['required', 'string']
        ];

        $this->validate( request(), $rules );

        Storage::disk('img_companies')->delete( request( 'img' ) );

        return $this->showMessage('Deleted Image', 200);
    }

    /**
     * Eliminar imagen del usuario
        ROLE
        ** Superadministradores
        ** Administradores
        ** Salesmanager
        ** Vendedores
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteCompanyImage( Company $company ) {
        Storage::disk('img_users')->delete( $company->url_img );
        $company->url_img = null;
        $company->save();
        return $this->showOne( $company );
    }

}
