<?php

namespace App\Http\Controllers\ParticularPointOfInterest;

use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\ParticularPointOfInterest;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;
use Illuminate\Database\Eloquent\Collection;

class ParticularPointOfInterestController extends ApiController
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $particularPoints;
        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $particularPoints = ParticularPointOfInterest::select('name_points', \DB::raw('count(*) as total'))->groupBy('name_points')->get();
        } else {
            $particularPoints = ParticularPointOfInterest::select('name_points', \DB::raw('count(*) as total'))
                                            ->groupBy('name_points')
                                            ->where('user_id', auth()->user()->id)
                                            ->get();
        }
        return $this->showAll(Collection::make($particularPoints));
    }

    public function getPoints()
    {
        $rules = [
            'name_points'    =>   ['required', 'string']
        ];
        $this->validate(request(), $rules);
        $points;
        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $points = ParticularPointOfInterest::where([
                'name_points' => request('name_points')
            ])->get();
        } else {
            $points = ParticularPointOfInterest::where([
                        'name_points' => request('name_points'),
                        'user_id' => auth()->user()->id,
                    ])->get();
        }
        return $this->showAll(Collection::make($points));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store()
    {
        $rules = [
            'name_points'    =>   ['required', 'string'],
            'file'           =>   ['required', 'string'],
            'colour'         =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $reader = \PhpOffice\PhpSpreadsheet\IOFactory::createReader("Xlsx");
        $spreadsheet = $reader->load(public_path().'/files/temporary/'.request('file'));
        $spreadsheet->setActiveSheetIndex(0);
        $numRows = $spreadsheet->setActiveSheetIndex(0)->getHighestRow();
        
        for($i = 2; $i <= $numRows; $i++){
            $name        =   $spreadsheet->getActiveSheet()->getCell('A'.$i)->getCalculatedValue();
            $address     =   $spreadsheet->getActiveSheet()->getCell('B'.$i)->getCalculatedValue();
            $latitude    =   $spreadsheet->getActiveSheet()->getCell('C'.$i)->getCalculatedValue();
            $longitude   =   $spreadsheet->getActiveSheet()->getCell('D'.$i)->getCalculatedValue();
            $colour      =   $spreadsheet->getActiveSheet()->getCell('E'.$i)->getCalculatedValue();
            
            $colour  = !empty($colour) ? $colour : request('colour');
            $name    = !empty($name) ? $name : 'name'.$i;
            $address = !empty($address) ? $address : 'address'.$i;

            ParticularPointOfInterest::create([
                'user_id'     => auth()->user()->id,
                'name_points' => request('name_points'),
                'name'        => $name,
                'address'     => $address,
                'latitude'    => $latitude,
                'longitude'   => $longitude,
                'colour'      => $colour
            ]);
        }

        $points = ParticularPointOfInterest::where(
            [
                'name_points' => request('name_points'),
                'user_id' => auth()->user()->id
            ]
        )->get();

        Storage::disk('temporaryFiles')->delete( request( 'file' ) );

        return $this->showOne(Collection::make($points));
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show()
    {
        $rules = [
            'name_points'   => ['required'],
            'name_points.*' => ['string']
        ];

        $this->validate(request(), $rules);

        $points;

        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $points = ParticularPointOfInterest::whereIn(
                'name_points', request('name_points')
            )->get();
        } else {
            $points = ParticularPointOfInterest::whereIn(                
                'name_points', request('name_points')
            )
            ->where( 'user_id', auth()->user()->id )
            ->get();
        }


        return $this->showOne($points);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function updatePoint(ParticularPointOfInterest $point)
    {
        if (!auth()->user()->hasRole('superadministrator') && !auth()->user()->hasRole('administrator')) {
            if ($point->user_id !== auth()->user()->id) {
                return $this->errorResponse('Error, you do not have access to this resource', 403);
            }
        }
        $rules = [
            'name_points'   => ['string'],
            'name'   => ['string'],
            'address' => ['string'],
            'colour' => ['string']
        ];

        $this->validate(request(), $rules);

        $point->fill(request()->only([
            'name_points',
            'name',
            'address',
            'latitude',
            'longitude',
            'colour'
        ]));

        $point->save();

        return $this->showOne($point);
    }

    /**
     * Store the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function storePoint()
    {
        $rules = [
            'name_points'   => ['required', 'string'],
            'name'   => ['required', 'string'],
            'address' => ['string'],
            'latitude' => ['required', 'string'],
            'longitude' => ['required', 'string'],
            'colour' => ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $point = ParticularPointOfInterest::create([
            'user_id' => auth()->user()->id,
            'name_points' => request('name_points'),
            'name' => request('name'),
            'address' => request('address'),
            'latitude' => request('latitude'),
            'longitude' => request('longitude'),
            'colour' => request('colour')
        ]);

        return $this->showOne( $point );
    }

    /**
     * Delete the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deletePoint(ParticularPointOfInterest $point)
    {
        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $point->delete();
        } else {
            if ($point->user_id !== auth()->user()->id) {
                return $this->errorResponse('Error, you do not have access to this resource', 403);
            } else {
                $point->delete();
            }
        }
        return $this->showOne($point);
    }

    /**
     * Delete the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deletePoints()
    {
        $rules = [
            'name_points'   => ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $points;

        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $points = ParticularPointOfInterest::where(
                'name_points', request('name_points')
            )->get();
        } else {
            $points = ParticularPointOfInterest::where(                
                [
                    'name_points' => request('name_points'),
                    'user_id' => auth()->user()->id
                ]
            )
            ->get();
        }

        $points->each(function ($point, $key) {
            $point->delete();
        });

        return $this->showMessage('Ok', 200);
    }

    /**
     * Subir archivo del usuario
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function uploadFile() {

        $file = request()->file( 'file' );

        $input = array( 'file' => $file );
        $rules = [ 'file' => ['required', 'mimes:xls,xlsx'] ];

        $validate = Validator::make($input, $rules);

        if ($validate->fails())
        {
            return $this->errorResponse('Only xls or xlsx', 422);
        }

        $data['file'] = request('file')->store('', 'temporaryFiles');

        return $this->responseUploadFile( $data, 201 );
    }

    /**
     * Eliminar imagen
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteFile() {
        $rules = [
            'file'        =>    ['required', 'string']
        ];

        $this->validate( request(), $rules );

        Storage::disk('temporaryFiles')->delete( request( 'file' ) );

        return $this->showMessage('Deleted File', 200);
    }
}
