<?php

namespace App\Http\Controllers\User;

use App\Company;
use App\Country;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\User;
use Spatie\Permission\Models\Role;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class UserController extends ApiController
{

    public function __construct()
    {
        parent::__construct();
        $this->middleware( 'can:users.index' )->only( 'index' );
        $this->middleware( 'can:users.usersAdminSales' )->only( 'usersAdminSales' );
        $this->middleware( 'can:users.usersSeller' )->only( 'usersSeller' );
        $this->middleware( 'can:users.getAllSuperAdministrators' )->only( 'getAllSuperAdministrators' );
        $this->middleware( 'can:users.getAllAdministrators' )->only( 'getAllAdministrators' );
        $this->middleware( 'can:users.getAllSalesManager' )->only( 'getAllSalesManager' );
        $this->middleware( 'can:users.getAllProvidersAdministrators' )->only( 'getAllProvidersAdministrators' );
        $this->middleware( 'can:users.getAllSellersCosts' )->only( 'getAllSellersCosts' );
        $this->middleware( 'can:users.getAllSellers' )->only( 'getAllSellers' );
        $this->middleware( 'can:users.getAllCustomersCosts' )->only( 'getAllCustomersCosts' );
        $this->middleware( 'can:users.getAllCustomers' )->only( 'getAllCustomers' );
        $this->middleware( 'can:users.getAllUsersSuspended' )->only( 'getAllUsersSuspended' );
        $this->middleware( 'can:users.store' )->only( 'store' );
        $this->middleware( 'can:users.search' )->only( 'search' );
        $this->middleware( 'can:users.destroy' )->only( 'destroy' );
    }

    /**
     * Metodo para listar todos los usuarios
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if (request()->get('per_page') === 'all') {
            return $this->showAll( User::all() );
        }
        $users = User::orderBy( 'id', 'desc' )
                     ->with('company:id,name_company')
                     ->with('roles')
                     ->paginate( 8);
        return $this->showAll( Collection::make($users) );
    }

    /**
     * Metodo para buscar un usuario
        ROLES
        ** Superadministrators
        ** Administrators
        ** Salesmanager
        ** Sellercosts Seller
     *
     * @return \Illuminate\Http\Response
     */
    public function search()
    {
        $rules = [
            'word'    =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $users;

        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $users = User::orderBy('id', 'DESC')
                         ->with('company:id,name_company')
                         ->with('roles')
                         ->orWhere('id', 'LIKE', '%'.request('word').'%')
                         ->orWhere('name', 'LIKE', '%'.request('word').'%')
                         ->orWhere('surnames', 'LIKE', '%'.request('word').'%')
                         ->orWhere('email', 'LIKE', '%'.request('word').'%')
                         ->paginate(8);
        }

        if (auth()->user()->hasRole('salesmanager')) {

            $users = User::where( 'admin_registry_id', '=', auth()->user()->id )
                            ->where( function($q){
                                $q->orWhere('id', 'LIKE', '%'.request('word').'%')
                                  ->orWhere('name', 'LIKE', '%'.request('word').'%')
                                  ->orWhere('surnames', 'LIKE', '%'.request('word').'%')
                                  ->orWhere('email', 'LIKE', '%'.request('word').'%');
                            })
                            ->with('company:id,name_company')
                            ->with('roles')
                            ->paginate( 8);
        }

        if (auth()->user()->hasRole('sellercosts') || auth()->user()->hasRole('seller')) {
            $users = User::where( 'user_registry_id', '=', auth()->user()->id )
                            ->where( function($q){
                                $q->orWhere('id', 'LIKE', '%'.request('word').'%')
                                  ->orWhere('name', 'LIKE', '%'.request('word').'%')
                                  ->orWhere('surnames', 'LIKE', '%'.request('word').'%')
                                  ->orWhere('email', 'LIKE', '%'.request('word').'%');
                            })
                            ->with('company:id,name_company')
                            ->with('roles')
                            ->paginate( 8);
        }

        return $this->showAll(Collection::make($users), 201);
    }


    /**
     * Listar todos los usuarios de un administrador ventas.
        ROLES
        ** Sales Manager
     *
     * @return \Illuminate\Http\Response
    */
    public function usersAdminSales() {
        if (request()->get('per_page') === 'all') {
            $users = auth()->user()->users_admin()
                     ->with('company:id,name_company')
                     ->with('roles')
                     ->get();
            return $this->showAll( Collection::make($users) );
        }
        $users = auth()->user()->users_admin()
                     ->with('company:id,name_company')
                     ->with('roles')
                     ->paginate( 8);
        return $this->showAll( Collection::make($users) );
    }

    /**
     * Listar todos los usuarios de un vendedor.
        ROLES
        ** Seller Cost
        ** Seller
     *
     * @return \Illuminate\Http\Response
    */
    public function usersSeller() {
        if (request()->get('per_page') === 'all') {
            $users = auth()->user()->customers;
            return $this->showAll( Collection::make($users) );
        }
        $users = auth()->user()->customers()
                     ->with('company:id,name_company')
                     ->with('roles')
                     ->paginate( 8);
        return $this->showAll( Collection::make($users) );
    }

    /**
     * Listar todos los superadministradores
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function getAllSuperAdministrators()
    {
        $data = User::select('*')
                    ->with('roles')
                    ->with('company:id,name_company')
                    ->whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 1);
                    })
                    ->distinct()
                    ->paginate(8);

        return $this->showAll( Collection::make($data) );
    }

    /**
     * Listar todos los administradores
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function getAllAdministrators()
    {
        if (request()->get('per_page') === 'all') {
            return $this->showAll( User::whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 2);
                    })->get() );
        }
        $data = User::select('*')
                    ->with('roles')
                    ->with('company:id,name_company')
                    ->whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 2);
                    })
                    ->distinct()
                    ->paginate(8);

        return $this->showAll( Collection::make($data) );
    }

    /**
     * Listar todos los administradores
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function getAllSalesManager()
    {
        $data = User::select('*')
                    ->with('roles')
                    ->with('company:id,name_company')
                    ->whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 3);
                    })
                    ->distinct()
                    ->paginate(8);

        return $this->showAll( Collection::make($data) );
    }

    /**
     * Listar todos los proveedores admins
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function getAllProvidersAdministrators()
    {
        $data = User::select('*')
                    ->with('roles')
                    ->with('company:id,name_company')
                    ->whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 4);
                    })
                    ->distinct()
                    ->paginate(8);


        return $this->showAll( Collection::make($data) );
    }

    /**
     * Listar todos los vendedores costos
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function getAllSellersCosts()
    {
        $data = User::select('*')
                    ->with('roles')
                    ->with('company:id,name_company')
                    ->whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 5);
                    })
                    ->distinct()
                    ->paginate(8);

        return $this->showAll( Collection::make($data) );
    }

    /**
     * Listar todos los vendedores sin costos
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function getAllSellers()
    {
        $data = User::select('*')
                    ->with('roles')
                    ->with('company:id,name_company')
                    ->whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 6);
                    })
                    ->distinct()
                    ->paginate(8);

        return $this->showAll( Collection::make($data) );
    }

    /**
     * Listar todos los clientes costos
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function getAllCustomersCosts()
    {
        $data = User::select('*')
                    ->with('roles')
                    ->with('company:id,name_company')
                    ->whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 7);
                    })
                    ->distinct()
                    ->paginate(8);

        return $this->showAll( Collection::make($data) );
    }

    /**
     * Listar todos los clientes sin costos
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function getAllCustomers()
    {
        $data = User::select('*')
                    ->with('roles')
                    ->with('company:id,name_company')
                    ->whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 8);
                    })
                    ->distinct()
                    ->paginate(8);

        return $this->showAll( Collection::make($data) );
    }

    /**
     * Listar todos los usuarios suspendidos
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
    */
    public function getAllUsersSuspended()
    {
        $data = User::select('*')
                    ->with('roles')
                    ->with('company:id,name_company')
                    ->whereHas('roles', function($query) {
                        $query->where('role_user.role_id', '=', 9);
                    })
                    ->distinct()
                    ->paginate(8);

        return $this->showAll( Collection::make($data) );
    }

    /**
     * Crear usuarios
        ROLES
        ** Superadministradores
        ** Administradores
        ** Administradores Ventas
        ** Vendedores Costos
        ** Vendedores
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store()
    {
        $rules = [
            'company_id'    =>   ['integer'],
            'country_id'    =>   ['required', 'integer'],
            'name'          =>   ['required', 'string', 'max:255'],
            'surnames'      =>   ['required', 'string', 'max:255'],
            'email'         =>   ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password'      =>   ['required', 'string', 'min:8'],
            'img_profile'   =>   ['string'],
            'role'          =>   ['required', 'integer']
        ];

        $this->validate(request(), $rules);

        if (request('company_id')) {
            if (!Company::find(request('company_id'))) {
                return $this->errorResponse('Non-existent company', 404);
            }
        }

        if (!Country::find(request('country_id'))) {
            return $this->errorResponse('Non-existent country', 404);
        }

        $user_registry_id = auth()->user()->id;

        $admin_registry_id;

        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator') || auth()->user()->hasRole('salesmanager')) {
            $admin_registry_id = auth()->user()->id;
        } else {
            $admin_registry_id = auth()->user()->admin_registry_id;
        }

        $fields = request()->all();

        $fields['user_registry_id'] = $user_registry_id;
        $fields['admin_registry_id'] = $admin_registry_id;

        $user = User::create($fields);

        // Verificamos que exista el role
        $role = Role::find(request('role'));
        $slug = 'suspended';

        if ($role) {
            $name_role = $role->name;

            if (auth()->user()->hasRole('salesmanager')) {
                if ($name_role == 'sellercosts' || $name_role == 'seller' || $name_role == 'customercosts' || $name_role == 'customer' || $name_role == 'suspended') {
                    $name_role = $name_role;
                } else {
                    $name_role = 'suspended';
                }
            } else if (auth()->user()->hasRole('sellercosts')) {
                if ($name_role == 'customercosts' || $name_role == 'customer' || $name_role == 'suspended') {
                    $name_role = $name_role;
                } else {
                    $name_role = 'suspended';
                }
            } else if (auth()->user()->hasRole('seller')) {
                if ($name_role == 'customer' || $name_role == 'suspended') {
                    $name_role = $name_role;
                } else {
                    $name_role = 'suspended';
                }
            } else {
                if (auth()->user()->hasRole('administrator') || auth()->user()->hasRole('superadministrator')) {
                    $name_role = $name_role;
                }
            }
        }

        $user->assignRoles($name_role);

        return $this->showOne($user, 201);
    }

    /**
     * Mostrar un usuario.
        ROLES
        ** TODOS LOS ROLES
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(User $user)
    {
        return $this->showOne($user);
    }

    /**
     * Actualizar usuarios
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(User $user)
    {

        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {

            if (auth()->user()->hasRole('administrator') && $user->hasRole('superadministrator')) {
                return $this->errorResponse('You have no control of this user', 403);
            }

            $user = $user;
        } else if (auth()->user()->hasRole('salesmanager')) {

            if ($user->id != auth()->user()->id) {
                if ($user->admin_registry_id != auth()->user()->id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        } else if (auth()->user()->hasRole('sellercosts') || auth()->user()->hasRole('seller')) {

            if ($user->id != auth()->user()->id) {
                if ($user->user_registry_id != auth()->user()->id) {
                    return $this->errorResponse('Error, you do not have access to this resource', 403);
                }
            }

        } else if (auth()->user()->hasRole('customercosts') || auth()->user()->hasRole('customer')) {

            if ($user->id != auth()->user()->id) {
                return $this->errorResponse('Error, you do not have access to this resource', 403);
            }

        } else {
            return $this->errorResponse('Error, you do not have access to this resource', 403);
        }

        $reglas = [
            'company_id' => ['integer'],
            'country_id' => ['integer'],
            'name' => ['string', 'max:255'],
            'surnames' => ['string', 'max:255'],
            'email' => ['string', 'email', 'max:255', 'unique:users,email,'.$user->id],
            'password' => ['string', 'min:6'],
            'img_profile' => ['string'],
            'role' => ['integer']
        ];

        $this->validate(request(), $reglas);

        $user->fill(request()->only(
            'name', 'surnames', 'img_profile', 'password'
        ));

        if (request('company_id')) {
            if (!Company::find(request('company_id'))) {
                return $this->errorResponse('Non-existent company', 404);
            }
            $user->company_id = request('company_id');
        }

        if (request('country_id')) {
            if (!Country::find(request('country_id'))) {
                return $this->errorResponse('Non-existent country', 404);
            }
            $user->country_id = request('country_id');
        }

        if (request('email') && $user->email != request('email')) {
            $user->email = request('email');
        }

        if (request('role')) {

            $role = Role::find(request('role'));
            $name_role = 'suspended';

            if ($role) {
                $name_role = $role->name;
                if (auth()->user()->hasRole('salesmanager')) {
                    if ($name_role == 'sellercosts' || $name_role == 'seller' || $name_role == 'customercosts' || $name_role == 'customer' || $name_role == 'suspended') {
                        $name_role = $name_role;
                    } else {
                        $name_role = 'suspended';
                    }
                } else if (auth()->user()->hasRole('sellercosts')) {
                    if ($name_role == 'customercosts' || $name_role == 'customer' || $name_role == 'suspended') {
                        $name_role = $name_role;
                    } else {
                        $name_role = 'suspended';
                    }
                } else if (auth()->user()->hasRole('seller')) {
                    if ($name_role == 'customer' || $name_role == 'suspended') {
                        $name_role = $name_role;
                    } else {
                        $name_role = 'suspended';
                    }
                } else {
                    if (auth()->user()->hasRole('administrator') || auth()->user()->hasRole('superadministrator')) {
                        $name_role = $name_role;
                    } else {
                        $name_role = 'suspended';
                    }
                }
            }

            if (count($user->roles) > 0){
                $user->removeRole($user->roles[0]->name);
            }

            $user->assignRole($name_role);
        }

        $user->save();

        return $this->showOne($user);
    }

    /**
     * Eliminar un usuario
        ROLE
        ** Superadministrator
        ** Administrator
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(User $user)
    {

        if ($user->hasRole('superadministrator')) {
            return $this->errorResponse('Error, you do not have access to this resource', 403);
        }

        $campaigns = $user->campaigns_sellers;

        foreach ($campaigns as $key => $campaign) {
            $campaign->polygons()->delete();
            $campaign->pointsOfInterest()->delete();
            $campaign->screens()->detach();
            $campaign->budget()->delete();
            $campaign->delete();
        }

        $assigned_campaigns = $user->campaigns_customers;

        foreach ($assigned_campaigns as $key => $campaign) {
            $campaign->customer_campaign_id = $campaign->user_registry_id;
            $campaign->save();
        }

        $customers = $user->customers;

        foreach ($customers as $key => $customer) {
            $customer->user_registry_id = 2;
            $customer->save();
        }

        if ($user->hasRole('administrator')) {
            $users_admins = $user->users_admin;
            foreach ($users_admins as $key => $user) {
                $user->admin_registry_id = 2;
                $user->save();
            }
        }

        $user->roles()->detach();

        $user->delete();

        return $this->showOne($user);
    }

    /**
     * Obtener información del usuario logeado
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function me() {

        $userAuthenticated = [
            'id'            => auth()->user()->id,
            'name'          => auth()->user()->name,
            'surnames'      => auth()->user()->surnames,
            'email'         => auth()->user()->email,
            'img_profile'   => auth()->user()->img_profile,
            'company_id'    => auth()->user()->company->id,
            'company'       => auth()->user()->company->name_company,
            'company_img'   => auth()->user()->company->url_img,
            'role_slug'     => auth()->user()->getRoleNames()[0]
        ];

        if (auth()->user()->roles[0]->slug === 'superadministrator' || auth()->user()->roles[0]->slug === 'administrator') {
            $userAuthenticated['team'] = auth()->user()->teams;
        }

        return $userAuthenticated;
    }

    /**
     * Cerrar sesión
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function logout() {
        request()->user()->token()->revoke();
        return $this->showMessage('Logout', 200);
    }


    /**
     * Subir imagen del usuario
        -- Sube la imagen, pero se guardara en el momento que se cree o actualize un usuario
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function uploadImage() {

        $img = request()->file( 'file' );

        $input = array( 'image' => $img );
        $rules = [ 'image' => ['required', 'image'] ];

        $validate = Validator::make($input, $rules);

        if ($validate->fails())
        {
            return $this->errorResponse('Only Images', 422);
        }

        $data['image'] = request('file')->store('', 'img_users');

        return $this->responseUploadFile( $data, 201 );
    }

    /**
     * Eliminar imagen
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteImage() {
        $rules = [
            'img'        =>    ['required', 'string']
        ];

        $this->validate( request(), $rules );

        Storage::disk('img_users')->delete( request( 'img' ) );

        return $this->showMessage('Deleted Image', 200);
    }

    /**
     * Eliminar imagen del usuario
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteUserImage( User $user ) {
        if (!auth()->user()->hasRole('superadministrator') && !auth()->user()->hasRole('administrator')) {
            if (auth()->user()->id != $user->id) {
                return $this->errorResponse('You have no control of this user', 403);
            }
        }
        Storage::disk('img_users')->delete( $user->img_profile );
        $user->img_profile = null;
        $user->save();
        return $this->showOne( $user );
    }

    /**
     * Veriricar si el usuario esta autenticado
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function authenticated() {
        return $this->showMessage( 'true', 200 );
    }

}
