<?php

namespace App\Http\Controllers\Wall;

use App\Company;
use App\Country;
use App\GalleryWall;
use App\Http\Controllers\ApiController;
use App\Http\Controllers\Controller;
use App\Provider;
use App\Reseller;
use App\Wall;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class WallController extends ApiController
{
    public function __construct()
    {
        parent::__construct();
        $this->middleware('can:walls.index')->only('index');
        $this->middleware('can:walls.getAllWallsDirectory')->only('getAllWallsDirectory');
        $this->middleware('can:walls.filteredWalls')->only('filteredWalls');
        $this->middleware('can:walls.getWallProvider')->only('getWallProvider');
        $this->middleware('can:walls.store')->only('store');
        $this->middleware('can:walls.show')->only('show');
        $this->middleware('can:walls.update')->only('update');
        $this->middleware('can:walls.uploadImage')->only('uploadImage');
        $this->middleware('can:walls.deleteImage')->only('deleteImage');
        $this->middleware('can:walls.deleteWallImage')->only('deleteWallImage');
        $this->middleware('can:walls.search')->only('search');
        $this->middleware('can:walls.searchWallProviderId')->only('searchWallProviderId');
        $this->middleware('can:walls.destroy')->only('destroy');
        $this->middleware('can:walls.searchWallsProviderId')->only('searchWallsProviderId');

    }

    /**
     * Metodo para listar todos las pantallas
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $wall = Wall::orderBy( 'id', 'desc' )
                     ->with('company:id,name_company')
                     ->with( 'country:id,country_name' )
                     ->with( 'images' )
                     ->paginate( 8 );
                     // ->makeHidden('local_currency');
        return $this->showAll( Collection::make($wall) );
    }

    /**
     * Metodo para buscar una pantalla
        ROLES
        ** Superadministrators
        ** Administrators
     *
     * @return \Illuminate\Http\Response
     */
    public function search()
    {
        $rules = [
            'word'    =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $wall = Wall::orderBy('id', 'DESC')
                         ->with('company:id,name_company')
                         ->with( 'country:id,country_name' )
                         ->orWhere('id', 'LIKE', '%'.request('word').'%')
                         ->orWhere('provider_wall_id', 'LIKE', '%'.request('word').'%')
                         ->orWhere('type', 'LIKE', '%'.request('word').'%')
                         ->paginate( 8 )
                         ->makeHidden('local_currency');

        return $this->showAll(Collection::make($wall), 201);
    }

    /**
     * Buscar una pantalla por id del proveedor
        ROLES
        ** Superadministradores
        ** Administradores
        ** Administradores Ventas
        ** Vendedores Costos
        ** Vendedores
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function searchWallProviderId()
    {
        $rules = [
            'provider_wall_id'    =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $wall;

        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $wall = Wall::orderBy('id', 'DESC')
                         ->with('company:id,name_company')
                         ->with( 'circuit:id,name' )
                         ->where('provider_wall_id', request('provider_wall_id'))
                         ->get()
                         ->makeHidden('local_currency');
            return $this->showAll(Collection::make($wall), 201);
        }

        if ( auth()->user()->hasRole('administratorprovider') ) {

            if (request('ooh') && !request('countries_ids')) {
                $wall = Wall::orderBy('id', 'DESC')
                         ->with('company:id,name_company')
                         ->with( 'circuit:id,name' )
                         ->where('provider_wall_id',  request('provider_wall_id'))
                         ->get()
                         ->makeHidden('local_currency');
            }
            return $this->showAll(Collection::make($wall), 201);
        }

        $wall = Wall::orderBy('id', 'DESC')
                         ->with( 'circuit:id,name' )
                         ->where('provider_wall_id',  request('provider_wall_id'))
                         ->get()
                         ->makeHidden('local_currency');
        return $this->showAll(Collection::make($wall), 201);
    }

    public function getAllWallsDirectory() {
        // With company es necesario para filtro administrador de empresas
        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {

            if (count(auth()->user()->company->countries) > 0) {

                $countriesIDs = auth()->user()->company->countries->map(function ($country) {
                    return strval($country->id);
                });

                $walls = Wall::orderBy('country_id', 'ASC')
                               ->where('disabled', false)
                               ->whereIn( 'country_id', $countriesIDs)
                               ->with( 'company:id,name_company' )
                               ->with( 'country' )
                               ->with( 'circuit:id,name' )
                               ->get()
                               ->makeHidden('local_currency');
              } else {
                $walls = Wall::orderBy('country_id', 'ASC')
                                   ->where('disabled', false)
                                   ->with( 'company:id,name_company' )
                                   ->with( 'country' )
                                   ->with( 'circuit:id,name' )
                                   ->get()
                                   ->makeHidden('local_currency');
              }

            return $this->showAll( Collection::make($walls) );
        }

        if ( auth()->user()->hasRole('administratorprovider') ) {
            // Revisar este metodo
            $walls = auth()->user()
                         ->company
                         ->walls
                         ->load( 'circuit:id,name' )
                         ->load( 'country' )
                         ->makeHidden('local_currency');
            return $this->showAll( Collection::make($walls) );
        }

        $walls = Wall::orderBy('country_id', 'ASC')
                            ->where('disabled', false)
                            ->with( 'circuit:id,name' )
                            ->with( 'country' )
                            ->get()
                            ->makeHidden('local_currency');

        return $this->showAll( Collection::make($walls) );
    }

    /**
     * Listar pantallas filtradas
        ROLES
        ** Todos los roles, menos el suspendido
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function filteredWalls()
    {
        $rules = [
                    'ooh' => ['in:outdoor,indoor'],
                    'countries_ids.*' => ['numeric'],
                    'states.*' => ['string']
                ];

        $this->validate(request(), $rules);

        $wall;

        $companies_ids = auth()->user()->company->suppliers->pluck('id');

        if (count($companies_ids) != 0) {
            if (!request('states')) {
                // With company es necesario para filtro administrador de empresas
                if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
                    if (request('ooh') && !request('countries_ids')) {
                        $wall = Wall::where(
                                [
                                    ['ooh', '=', request('ooh')],
                                    ['disabled', '=', false],
                                    ['country_id', '!=', 5]
                                ]
                            )
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'company:id,name_company' )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && !request('ooh')) {
                        $wall = Wall::where(
                                [
                                    ['disabled', '=', false],
                                    ['country_id', '!=', 5]
                                ]
                            )
                            ->whereIn('country_id', request('countries_ids'))
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'company:id,name_company' )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && request('ooh')) {
                        $wall = Wall::where(
                                [
                                    ['ooh', '=', request('ooh')],
                                    ['disabled', '=', false],
                                    ['country_id', '!=', 5]
                                ]
                            )
                            ->whereIn( 'country_id', request('countries_ids'))
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'company:id,name_company' )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    if (!request('countries_ids') && !request('ooh')) {

                        if (count(auth()->user()->company->countries) > 0) {
                            $countriesIDs = auth()->user()->company->countries->map(function ($country) {
                                return strval($country->id);
                            });

                            $wall = Wall::where(
                                    [
                                        ['disabled', '=', false],
                                        ['country_id', '!=', 5]
                                    ]
                                )
                                ->whereIn( 'country_id', $countriesIDs)
                                ->whereIn( 'company_id', $companies_ids )
                                ->with( 'company:id,name_company' )
                                ->with( 'circuit:id,name' )
                                ->get()
                                ->makeHidden('local_currency');
                        } else {
                            $wall = Wall::where(
                                    [
                                        ['disabled', '=', false],
                                        ['country_id', '!=', 5]
                                    ]
                                )
                                ->whereIn( 'company_id', $companies_ids )
                                ->with( 'company:id,name_company' )
                                ->with( 'circuit:id,name' )
                                ->get()
                                ->makeHidden('local_currency');
                        }
                    }

                    return $this->showAll( Collection::make($wall) );
                }

                if ( auth()->user()->hasRole('administratorprovider') ) {

                    if (request('ooh') && !request('countries_ids')) {
                        $wall = Wall::where(
                                [
                                    ['ooh', '=', request('ooh')],
                                    ['company_id', '=', auth()->user()->company_id],
                                    ['country_id', '!=', 5]
                                ]
                            )
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && !request('ooh')) {
                        $wall = Wall::where(
                                [
                                    ['company_id', '=', auth()->user()->company_id],
                                    ['country_id', '!=', 5]
                                ]
                            )
                            ->whereIn('country_id', request('countries_ids'))
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && request('ooh')) {
                        $wall = Wall::where(
                                [
                                    ['ooh', '=', request('ooh')],
                                    ['company_id', '=', auth()->user()->company_id],
                                    ['country_id', '!=', 5]
                                ]
                            )
                            ->whereIn('country_id', request('countries_ids'))
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    // Revisar este metodo
                    if (!request('countries_ids') && !request('ooh')) {
                        $wall = auth()->user()
                            ->company
                            ->walls
                            ->whereHas('walls' ,function($q) use($request){
                                $q->where(
                                    [
                                        ['country_id', '!=', 5]
                                    ]
                                );
                            })
                            ->load( 'circuit:id,name' )
                            ->makeHidden('local_currency');
                    }

                    return $this->showAll( Collection::make($wall) );
                }

                if (request('ooh') && !request('countries_ids')) {
                    $wall = Wall::where(
                            [
                                ['ooh', '=', request('ooh')],
                                ['disabled', '=', false],
                                ['country_id', '!=', 5]
                            ]
                        )
                        ->whereIn( 'company_id', $companies_ids )
                        ->with( 'circuit:id,name' )
                        ->get()
                        ->makeHidden('local_currency');
                }

                if (request('countries_ids') && !request('ooh')) {
                    $wall = Wall::where(
                            [
                                ['disabled', '=', false],
                                ['country_id', '!=', 5]
                            ]
                        )
                        ->whereIn('country_id', request('countries_ids'))
                        ->whereIn( 'company_id', $companies_ids )
                        ->with( 'circuit:id,name' )
                        ->get()
                        ->makeHidden('local_currency');
                }

                if (request('countries_ids') && request('ooh')) {
                    $wall = Wall::where(
                            [
                                ['ooh', '=', request('ooh')],
                                ['disabled', '=', false],
                                ['country_id', '!=', 5]
                            ]
                        )
                        ->whereIn( 'country_id', request('countries_ids'))
                        ->whereIn( 'company_id', $companies_ids )
                        ->with( 'circuit:id,name' )
                        ->get()
                        ->makeHidden('local_currency');
                }

                if (!request('countries_ids') && !request('ooh')) {
                    $wall = Wall::where(
                        [
                            ['disabled', '=', false],
                            ['country_id', '!=', 5]
                        ]
                        )
                        ->whereIn( 'company_id', $companies_ids )
                        ->with( 'circuit:id,name' )
                        ->get()
                        ->makeHidden('local_currency');
                }
            } else {

                if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {

                    if (request('countries_ids') && !request('ooh')) {
                        $wall = Wall::where(
                                [
                                    'disabled' => false
                                ]
                            )
                            ->whereIn('country_id', request('countries_ids'))
                            ->whereIn('state', request('states'))
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'company:id,name_company' )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && request('ooh')) {
                        $wall = Wall::where(
                                [
                                    'ooh' => request('ooh'),
                                    'disabled' => false
                                ]
                            )
                            ->whereIn( 'country_id', request('countries_ids'))
                            ->whereIn('state', request('states'))
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'company:id,name_company' )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    return $this->showAll( Collection::make($wall) );
                }

                if ( auth()->user()->hasRole('administratorprovider') ) {

                    if (request('countries_ids') && !request('ooh')) {
                        $wall = Wall::where(
                                [
                                    'company_id' => auth()->user()->company_id,
                                ]
                            )
                            ->whereIn('country_id', request('countries_ids'))
                            ->whereIn('state', request('states'))
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && request('ooh')) {
                        $wall = Wall::where(
                                [
                                    'ooh' => request('ooh'),
                                    'company_id' => auth()->user()->company_id
                                ]
                            )
                            ->whereIn('country_id', request('countries_ids'))
                            ->whereIn('state', request('states'))
                            ->whereIn( 'company_id', $companies_ids )
                            ->with( 'circuit:id,name' )
                            ->get()
                            ->makeHidden('local_currency');
                    }

                    return $this->showAll( Collection::make($wall) );
                }

                if (request('countries_ids') && !request('ooh')) {
                    $wall = Wall::where(
                            [
                                'disabled' => false
                            ]
                        )
                        ->whereIn('country_id', request('countries_ids'))
                        ->whereIn('state', request('states'))
                        ->whereIn( 'company_id', $companies_ids )
                        ->with( 'circuit:id,name' )
                        ->get()
                        ->makeHidden('local_currency');
                }

                if (request('countries_ids') && request('ooh')) {
                    $wall = Wall::where(
                            [
                                'ooh' => request('ooh'),
                                'disabled' => false
                            ]
                        )
                        ->whereIn( 'country_id', request('countries_ids'))
                        ->whereIn('state', request('states'))
                        ->whereIn( 'company_id', $companies_ids )
                        ->with( 'circuit:id,name' )
                        ->get()
                        ->makeHidden('local_currency');
                }

            }
        } else {

            if (!request('states')) {
                // With company es necesario para filtro administrador de empresas
                if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
                    if (request('ooh') && !request('countries_ids')) {
                        $wall = Wall::where(
                                                [
                                                    ['ooh', '=', request('ooh')],
                                                    ['disabled', '=', false],
                                                    ['country_id', '!=', 5]
                                                ]
                                            )
                                            ->with( 'company:id,name_company' )
                                            ->with( 'circuit:id,name' )
                                            ->get()
                                            ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && !request('ooh')) {
                        $wall = Wall::where(
                                                    [
                                                        ['disabled', '=', false],
                                                        ['country_id', '!=', 5]
                                                    ]
                                                )
                                                ->whereIn('country_id', request('countries_ids'))
                                                ->with( 'company:id,name_company' )
                                                ->with( 'circuit:id,name' )
                                                ->get()
                                                ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && request('ooh')) {
                        $wall = Wall::where(
                                                    [
                                                        ['ooh', '=', request('ooh')],
                                                        ['disabled', '=', false],
                                                        ['country_id', '!=', 5]
                                                    ]
                                                )
                                                ->whereIn( 'country_id', request('countries_ids'))
                                                ->with( 'company:id,name_company' )
                                                ->with( 'circuit:id,name' )
                                                ->get()
                                                ->makeHidden('local_currency');
                    }

                    if (!request('countries_ids') && !request('ooh')) {

                      if (count(auth()->user()->company->countries) > 0) {
                        $countriesIDs = auth()->user()->company->countries->map(function ($country) {
                            return strval($country->id);
                        });

                        $wall = Wall::where(
                                          [
                                              ['disabled', '=', false],
                                              ['country_id', '!=', 5]
                                          ]
                                       )
                                       ->whereIn( 'country_id', $countriesIDs)
                                       ->with( 'company:id,name_company' )
                                       ->with( 'circuit:id,name' )
                                       ->get()
                                       ->makeHidden('local_currency');
                      } else {
                        $wall = Wall::where(
                                              [
                                                  ['disabled', '=', false],
                                                  ['country_id', '!=', 5]
                                              ]
                                           )
                                           ->with( 'company:id,name_company' )
                                           ->with( 'circuit:id,name' )
                                           ->get()
                                           ->makeHidden('local_currency');
                      }
                    }

                    return $this->showAll( Collection::make($wall) );
                }

                if ( auth()->user()->hasRole('administratorprovider') ) {

                    if (request('ooh') && !request('countries_ids')) {
                        $wall = Wall::where(
                                                    [
                                                        ['ooh', '=', request('ooh')],
                                                        ['company_id', '=', auth()->user()->company_id],
                                                        ['country_id', '!=', 5]
                                                    ]
                                                )
                                                ->with( 'circuit:id,name' )
                                                ->get()
                                                ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && !request('ooh')) {
                        $wall = Wall::where(
                                            [
                                                ['company_id', '=', auth()->user()->company_id],
                                                ['country_id', '!=', 5]
                                            ]
                                         )
                                         ->whereIn('country_id', request('countries_ids'))
                                         ->with( 'circuit:id,name' )
                                         ->get()
                                         ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && request('ooh')) {
                        $wall = Wall::where(
                                                    [
                                                        ['ooh', '=', request('ooh')],
                                                        ['company_id', '=', auth()->user()->company_id],
                                                        ['country_id', '!=', 5]
                                                    ]
                                                )
                                          ->whereIn('country_id', request('countries_ids'))
                                          ->with( 'circuit:id,name' )
                                          ->get()
                                          ->makeHidden('local_currency');
                    }

                    // Revisar este metodo
                    if (!request('countries_ids') && !request('ooh')) {
                        $wall = auth()->user()
                                         ->company
                                         ->walls
                                         ->whereHas('walls' ,function($q) use($request){
                                            $q->where(
                                                [
                                                    ['country_id', '!=', 5]
                                                ]
                                            );
                                         })
                                         ->load( 'circuit:id,name' )
                                         ->makeHidden('local_currency');
                    }

                    return $this->showAll( Collection::make($wall) );
                }

                if (request('ooh') && !request('countries_ids')) {
                    $wall = Wall::where(
                                                [
                                                    ['ooh', '=', request('ooh')],
                                                    ['disabled', '=', false],
                                                    ['country_id', '!=', 5]
                                                ]
                                            )
                                            ->with( 'circuit:id,name' )
                                            ->get()
                                            ->makeHidden('local_currency');
                }

                if (request('countries_ids') && !request('ooh')) {
                    $wall = Wall::where(
                                                [
                                                    ['disabled', '=', false],
                                                    ['country_id', '!=', 5]
                                                ]
                                            )
                                            ->whereIn('country_id', request('countries_ids'))
                                            ->with( 'circuit:id,name' )
                                            ->get()
                                            ->makeHidden('local_currency');
                }

                if (request('countries_ids') && request('ooh')) {
                    $wall = Wall::where(
                                                [
                                                    ['ooh', '=', request('ooh')],
                                                    ['disabled', '=', false],
                                                    ['country_id', '!=', 5]
                                                ]
                                            )
                                            ->whereIn( 'country_id', request('countries_ids'))
                                            ->with( 'circuit:id,name' )
                                            ->get()
                                            ->makeHidden('local_currency');
                }

                if (!request('countries_ids') && !request('ooh')) {
                    $wall = Wall::where(
                                        [
                                            ['disabled', '=', false],
                                            ['country_id', '!=', 5]
                                        ]
                                    )->with( 'circuit:id,name' )
                                    ->get()
                                    ->makeHidden('local_currency');
                }
            } else {

                if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {

                    if (request('countries_ids') && !request('ooh')) {
                        $wall = Wall::where(
                                                    [
                                                        'disabled' => false
                                                    ]
                                                )
                                                ->whereIn('country_id', request('countries_ids'))
                                                ->whereIn('state', request('states'))
                                                ->with( 'company:id,name_company' )
                                                ->with( 'circuit:id,name' )
                                                ->get()
                                                ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && request('ooh')) {
                        $wall = Wall::where(
                                                    [
                                                        'ooh' => request('ooh'),
                                                        'disabled' => false
                                                    ]
                                                )
                                                ->whereIn( 'country_id', request('countries_ids'))
                                                ->whereIn('state', request('states'))
                                                ->with( 'company:id,name_company' )
                                                ->with( 'circuit:id,name' )
                                                ->get()
                                                ->makeHidden('local_currency');
                    }

                    return $this->showAll( Collection::make($wall) );
                }

                if ( auth()->user()->hasRole('administratorprovider') ) {

                    if (request('countries_ids') && !request('ooh')) {
                        $wall = Wall::where(
                                            [
                                                'company_id' => auth()->user()->company_id,
                                            ]
                                         )
                                         ->whereIn('country_id', request('countries_ids'))
                                         ->whereIn('state', request('states'))
                                         ->with( 'circuit:id,name' )
                                         ->get()
                                         ->makeHidden('local_currency');
                    }

                    if (request('countries_ids') && request('ooh')) {
                        $wall = Wall::where(
                                                    [
                                                        'ooh' => request('ooh'),
                                                        'company_id' => auth()->user()->company_id
                                                    ]
                                                )
                                          ->whereIn('country_id', request('countries_ids'))
                                          ->whereIn('state', request('states'))
                                          ->with( 'circuit:id,name' )
                                          ->get()
                                          ->makeHidden('local_currency');
                    }

                    return $this->showAll( Collection::make($wall) );
                }

                if (request('countries_ids') && !request('ooh')) {
                    $wall = Wall::where(
                                                [
                                                    'disabled' => false
                                                ]
                                            )
                                            ->whereIn('country_id', request('countries_ids'))
                                            ->whereIn('state', request('states'))
                                            ->with( 'circuit:id,name' )
                                            ->get()
                                            ->makeHidden('local_currency');
                }

                if (request('countries_ids') && request('ooh')) {
                    $wall = Wall::where(
                                                [
                                                    'ooh' => request('ooh'),
                                                    'disabled' => false
                                                ]
                                            )
                                            ->whereIn( 'country_id', request('countries_ids'))
                                            ->whereIn('state', request('states'))
                                            ->with( 'circuit:id,name' )
                                            ->get()
                                            ->makeHidden('local_currency');
                }

            }
        }

        return $this->showAll( Collection::make($wall) );
    }

    /**
     * Obtener el numero de pantalla de un establecimiento
        ROLES
        ** todos
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    public function getNumberWallsNameBusiness()
    {
      $rules = ['business_name' => ['required', 'string']];
      $this->validate(request(), $rules);

      $numberWalls = Wall::select('business_name', DB::raw('count(*) as total'))
                                            ->groupBy('business_name')
                                            ->where('business_name', mb_strtolower(request('business_name')))
                                            ->get();
      return $this->showOne(['number_walls' => $numberWalls], 200);
    }

    /**
     * Listar pantallas de un proveedor
        ROLES
        ** Admin Provider
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function getWallProvider( Company $company )
    {
        $wall = $company->walls()
                           ->with( 'circuit:id,name' )
                           ->with( 'country:id,country_name' )
                           ->with( 'company:id,name_company' )
                           ->paginate( 8 );
        return $this->showAll( Collection::make($wall) );
    }

    /**
     * Crear pantalla
        ROLES
        ** Superadministradores
        ** Administradores
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store()
    {
        $rules = [
            'company_id'            =>      ['required', 'integer'],
            'country_id'            =>      ['required', 'integer'],
            'provider_wall_id'      =>      ['required', 'string', 'min:5'],
            'ooh'                   =>      ['required', 'in:'.Wall::INDOOR.','.Wall::OUTDOOR],
            'type'                  =>      ['required', 'string'],
            'country'               =>      ['string'],
            'state'                 =>      ['required', 'string'],
            'town_hall_delegation'  =>      ['string'],
            'suburb'                =>      ['string'],
            'street'                =>      ['string'],
            'location_number'       =>      ['string'],
            'postal_code'           =>      ['string'],
            'latitude'              =>      ['required', 'string'],
            'longitude'             =>      ['required', 'string'],
            'latitude_view'         =>      ['string'],
            'longitude_view'        =>      ['string'],
            'vertical_horizontal'   =>      ['required', 'in:'.Wall::VERTICAL.','.Wall::HORIZONTAL],
            'measured_in_meters'    =>      ['string'],
            'average_speed'         =>      ['string'],
            'photographic_witness'  =>      ['required', 'in:si,no'],
            'local_currency'        =>      ['numeric'],
            'description'           =>      ['string'],
            'purchase_remarks'      =>      ['string'],
            'witness_time'          =>      ['integer'],
            'url_img'               =>      ['string'],
            'business_type'         =>      ['string'],
            'business_name'         =>      ['string'],
            'tags'                  =>      ['string'],
            'black_list'            =>      ['string'],
            'circuit_id'            =>      ['integer'],
            'material_art'          =>      ['string'],
            'number_faces'          =>      ['integer'],
            'light'                 =>      ['in:'.true.','.false],
            'type_installation'     =>      ['string'],
            'production_cost'       =>      ['numeric'],
            'installation_cost'     =>      ['numeric'],
            'third_party_printing'  =>      ['in:'.true.','.false],
            'requires_art'          =>      ['string'],
            'art_format'            =>      ['string'],
            'quality_art'           =>      ['string'],
            'art_printing_time'     =>      ['string'],
            'art_installation_time' =>      ['string'],
            'additional_requirements' =>    ['string'],
            'availability_flag'       =>    ['in:'.true.','.false]
        ];

        $this->validate(request(), $rules);

        $company = Company::find(request('company_id'));

        if (!$company) {
            return $this->errorResponse('Non-existent company', 404);
        } else {
            if ($company->role != 'PROVIDER') {
                return $this->errorResponse('The company is not a type of supplier', 404);
            }
        }

        if (!Country::find(request('country_id'))) {
            return $this->errorResponse('Non-existent country', 404);
        }

        if (request('circuit_id')) {
            if (!Circuit::find(request('circuit_id'))) {
                return $this->errorResponse('Non-existent circuit', 404);
            }
        }

        $wall = Wall::create(request()->all());

        return $this->showOne($wall, 201);
    }

    /**
     * Mostrar una pantalla
        ROLES
        ** Todos menos suspendido
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show(Wall $wall)
    {
        $wall = $wall->load('country')->load('circuit')->load('images');

        if (auth()->user()->hasRole('administratorprovider')) {
            if (auth()->user()->company_id != $wall->company_id) {
                return $this->errorResponse('Error, you do not have access to this resource', 422);
            }
            $wall = $wall->makeHidden('local_currency');
            return $this->showOne($wall);
        }

        if (auth()->user()->hasRole('seller') || auth()->user()->hasRole('customer')) {
            return $this->showOne($wall->makeHidden('local_currency'));
        }

        $percentage_provider = $wall->company()->select('cc.discount', 'cc.increase')
            ->join('company_country as cc','cc.company_id','=','companies.id')
            ->where('cc.country_id', $wall->country_id)
            ->get();

        $percentage_seller = DB::table('company_country')
            ->select('company_id', 'country_id', 'discount', 'increase')
            ->where('company_id', auth()->user()->company ? auth()->user()->company->id : null)
            ->get();

        $increase = $percentage_provider[0]->increase / 100;
        $discount = $percentage_provider[0]->discount / 100;

        $multiplierProvider = Provider::where( 'company_id', $wall->company_id )->first();
        $multiplierProvider = $multiplierProvider ? $multiplierProvider->multiplier : 1;

        $multiplierReseller = Reseller::where( 'company_id', auth()->user()->company ? auth()->user()->company->id : null )->first();
        $multiplierReseller  = $multiplierReseller ? $multiplierReseller->multiplier : 1;

        $price = $wall->local_currency * $multiplierProvider * $multiplierReseller;
        $price_discount = $price - $price * $discount;
        $price_increase = $price_discount + $price_discount * $increase;

        $price = $price_increase;

        $wall->price = $price;
        $wall->price_usd = $price / $wall->country->dollar_change;

        if (count($percentage_seller) > 0) {
            foreach ($percentage_seller as $key => $pS) {
                if($pS->country_id == $wall->country_id) {
                    $price_seller_discount = $wall->price - ($wall->price * ($pS->discount / 100));
                    $price_seller_increase = $price_seller_discount * (($pS->increase / 100) + 1);
                    $wall->price = $price_seller_increase;
                    $wall->price_usd = $price_seller_increase / $wall->country->dollar_change;
                }
            }
        }

        if (auth()->user()->hasRole('superadministrator') || auth()->user()->hasRole('administrator')) {
            $wall->discount = $discount * 100;
            $wall->increase = $increase * 100;
            $wall->multiplier_provider = $multiplierProvider;
            $wall->load('company');
        }

        if (auth()->user()->hasRole('salesmanager') || auth()->user()->hasRole('sellercosts') || auth()->user()->hasRole('customercosts')) {
            $wall->makeHidden('local_currency');
        }

        return $this->showOne($wall);
    }

    /**
     * Actualizar pantalla
        ROLES
        ** Superadministradores
        ** Administradores
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function update(Wall $wall)
    {

        if (!auth()->user()->hasRole('superadministrator')) {
            if (!$this->validateTeamPermissions($wall->country_id)) {
                return $this->errorResponse('Error, you do not have access to this resource', 403);
            }
        }

        $rules = [
            'company_id' => ['integer'],
            'country_id' => ['integer'],
            'provider_wall_id' => ['string', 'min:5'],
            'ooh' => ['in:'.Wall::INDOOR.','.Wall::OUTDOOR],
            'type' => ['string'],
            'country' => ['string'],
            'state' => ['string'],
            'town_hall_delegation' => ['string'],
            'suburb' => ['string'],
            'street' => ['string'],
            'location_number' => ['string'],
            'postal_code' => ['string'],
            'measured_in_meters' => ['string'],
            'average_speed' => ['string'],
            'photographic_witness' => ['in:si,no'],
            'local_currency' => ['numeric'],
            'description' => ['string'],
            'purchase_remarks' => ['string'],
            'witness_time' => ['integer'],
            'url_img' => ['string'],
            'business_type' => ['string'],
            'business_name' => ['string'],
            'tags' => ['string'],
            'black_list' => ['string'],
            'circuit_id' => ['integer'],
            'disabled' => ['in:'.true.','.false],
            'material_art' => ['string'],
            'number_faces' => ['integer'],
            'light' => ['in:'.true.','.false],
            'type_installation' => ['string'],
            'production_cost' => ['numeric'],
            'installation_cost' => ['numeric'],
            'third_party_printing' => ['in:'.true.','.false],
            'requires_art' => ['string'],
            'art_format' => ['string'],
            'quality_art' => ['string'],
            'additional_requirements' => ['string'],
            'availability_flag' => ['in:'.true.','.false]
        ];

        $this->validate(request(), $rules);

        $wall->fill(request()->only(
            'provider_wall_id',
            'ooh',
            'type',
            'country',
            'state',
            'town_hall_delegation',
            'suburb',
            'street',
            'location_number',
            'postal_code',
            'latitude',
            'longitude',
            'latitude_view',
            'longitude_view',
            'vertical_horizontal',
            'measured_in_meters',
            'average_monthly_impacts',
            'average_speed',
            'photographic_witness',
            'local_currency',
            'description',
            'purchase_remarks',
            'witness_time',
            'url_img',
            'business_type',
            'business_name',
            'tags',
            'black_list',
            'circuit_id',
            'disabled',
            'material_art',
            'number_faces',
            'light',
            'type_installation',
            'sales_model',
            'production_cost',
            'installation_cost',
            'third_party_printing',
            'requires_art',
            'art_format',
            'quality_art',
            'art_printing_time',
            'art_installation_time',
            'additional_requirements',
            'availability_flag'
        ));

        if (request('company_id')) {
            $company = Company::find(request('company_id'));
            if (!$company) {
                return $this->errorResponse('Non-existent company', 404);
            } else {
                if ($company->role != 'provider') {
                    return $this->errorResponse('The company is not a type of supplier', 404);
                }
            }
            $wall->company_id = request('company_id');
        }

        if (request('country_id')) {
            if (!Country::find(request('country_id'))) {
                return $this->errorResponse('Non-existent country', 404);
            }
            $wall->country_id = request('country_id');
        }

        if (request('circuit_id')) {
            if (!Circuit::find(request('circuit_id'))) {
                return $this->errorResponse('Non-existent circuit', 404);
            }
            $wall->circuit_id = request('country_id');
        }

        $wall->save();

        return $this->showOne($wall);
    }

    /**
     * Eliminar un usuario
        ROLE
        ** Superadministrator
        ** Administrator
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Wall $wall)
    {
        if (!auth()->user()->hasRole('superadministrator')) {
            if (!$this->validateTeamPermissions($wall->country_id)) {
                return $this->errorResponse('Error, you do not have access to this resource', 403);
            }
        }
        $wall->campaigns()->detach();
        $wall->delete();
        return $this->showOne($wall);
    }

    /**
     * Subir imagen del usuario
        -- Sube la imagen, pero se guardara en el momento que se cree o actualize un usuario
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function uploadImage(Company $company) {
        $img = request()->file( 'file' );

        $input = [
            'image' => $img
        ];

        $rules = [
            'image' => ['required', 'image']
        ];

        $validate = Validator::make($input, $rules);

        if ($validate->fails())
        {
            return $validate->errors();
        }

        if ($company->role != 'provider') {
            return $this->errorResponse('The company is not a type of supplier', 404);
        }

        $data['image'] = request('file')->store('/'.$company->id, 'img_walls');

        return $this->responseUploadFile( $data, 201 );
    }

    /**
     * Eliminar imagen
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteImage() {
        $rules = [
            'img'        =>    ['required', 'string']
        ];

        $this->validate( request(), $rules );

        Storage::disk('img_walls')->delete( request( 'img' ) );

        return $this->showMessage('Deleted Image', 200);
    }

    /**
     * Eliminar imagen del usuario
        ROLE
        ** TODOS LOS ROLES
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function deleteWallImage( Wall $wall ) {
        Storage::disk('img_walls')->delete( request( $wall->url_img ) );
        $wall->url_img = null;
        $wall->save();
        return $this->showOne( $wall );
    }

    public function deleteWallImageGallery(GalleryWall $galleryWall) {
        Storage::disk('img_walls_gallery')->delete( $galleryWall->img );
        $galleryWall->delete();
        return $this->showOne($galleryWall);
    }

    /**
     * Metodo para buscar una pantalla
    ROLES
     ** Superadministrators
     ** Administrators
     *
     * @return \Illuminate\Http\Response
     */
    public function searchWallsProviderId()
    {
        $rules = [
            'word'    =>   ['required', 'string']
        ];

        $this->validate(request(), $rules);

        $walls = Wall::select('provider_wall_id')->orderBy('id', 'DESC')
            ->where('provider_wall_id', 'LIKE', '%'.request('word').'%')
            ->take( 20 )
            ->get();

        return $this->showAll( Collection::make($walls) );
    }
}
